@@ -283,6 +283,7 @@ webkit_pref_callback_font_family (GSettings *settings,
   g_free (value);
 }
 
+#ifndef HAVE_WEBKIT2
 /* Part of this code taken from libsoup (soup-session.c) */
 static gchar *
 build_accept_languages_header (GArray *languages)
@@ -294,7 +295,7 @@ build_accept_languages_header (GArray *languages)
 
   g_return_val_if_fail (languages != NULL, NULL);
 
-  /* Calculate deltas for the quality values */
+  /* Calculate deltas for the quality values. */
   if (languages->len < 10)
     delta = 10;
   else if (languages->len < 20)
@@ -302,18 +303,18 @@ build_accept_languages_header (GArray *languages)
   else
     delta = 1;
 
-  /* Set quality values for each language */
-  langs = (gchar **) languages->data;
+  /* Set quality values for each language. */
+  langs = (gchar **)languages->data;
   for (i = 0; langs[i] != NULL; i++) {
-    gchar *lang = (gchar *) langs[i];
+    gchar *lang = (gchar *)langs[i];
     gint quality = 100 - i * delta;
 
     if (quality > 0 && quality < 100) {
       gchar buf[8];
-      g_ascii_formatd (buf, 8, "%.2f", quality/100.0);
+      g_ascii_formatd (buf, 8, "%.2f", quality / 100.0);
       langs[i] = g_strdup_printf ("%s;q=%s", lang, buf);
     } else {
-      /* Just dup the string in this case */
+      /* Just dup the string in this case. */
       langs[i] = g_strdup (lang);
     }
     g_free (lang);
@@ -325,6 +326,7 @@ build_accept_languages_header (GArray *languages)
 
   return langs_str;
 }
+#endif
 
 /* Based on Christian Persch's code from gecko backend of epiphany
    (old transform_accept_languages_list() function) */
@@ -333,18 +335,15 @@ webkit_pref_callback_accept_languages (GSettings *settings,
                                        char *key,
                                        gpointer data)
 {
-#ifdef HAVE_WEBKIT2
-  /* TODO: Languages */
-#else
+#ifndef HAVE_WEBKIT2
   SoupSession *session;
+  char *webkit_pref = data;
+  char *langs_str;
+#endif
   GArray *array;
   char **languages;
-  char *langs_str;
-  char *webkit_pref;
   int i;
 
-  webkit_pref = data;
-
   languages = g_settings_get_strv (settings, key);
 
   array = g_array_new (TRUE, FALSE, sizeof (char *));
@@ -360,16 +359,21 @@ webkit_pref_callback_accept_languages (GSettings *settings,
 
   ephy_langs_sanitise (array);
 
+#ifdef HAVE_WEBKIT2
+  webkit_web_context_set_preferred_languages (webkit_web_context_get_default (),
+                                              (const char * const *)array->data);
+#else
   langs_str = build_accept_languages_header (array);
 
   /* Update Soup session */
   session = webkit_get_default_session ();
   g_object_set (G_OBJECT (session), webkit_pref, langs_str, NULL);
 
-  g_strfreev (languages);
   g_free (langs_str);
-  g_array_free (array, TRUE);
 #endif
+
+  g_strfreev (languages);
+  g_array_free (array, TRUE);
 }
 
 
