@@ -30,6 +30,8 @@ typedef enum {
   /* WRITE */
   SET_URL_TITLE,
   SET_URL_ZOOM_LEVEL,
+  SET_URL_HIDDEN,
+  SET_URL_THUMBNAIL_TIME,
   ADD_VISIT,
   ADD_VISITS,
   DELETE_URLS,
@@ -49,6 +51,9 @@ typedef enum {
 enum {
   VISIT_URL,
   CLEARED,
+  URL_TITLE_CHANGED,
+  URL_DELETED,
+  HOST_DELETED,
   LAST_SIGNAL
 };
 
@@ -172,6 +177,37 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                   G_TYPE_NONE,
                   0);
 
+  signals[URL_TITLE_CHANGED] =
+    g_signal_new ("url-title-changed",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_NONE,
+                  2,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
+  signals[URL_DELETED] =
+    g_signal_new ("url-deleted",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__STRING,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
+  signals[HOST_DELETED] =
+    g_signal_new ("host-deleted",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__STRING,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
   g_object_class_install_property (gobject_class,
                                    PROP_HISTORY_FILENAME,
                                    g_param_spec_string ("history-filename",
@@ -425,13 +461,15 @@ ephy_history_service_execute_job_callback (gpointer data)
 {
   EphyHistoryServiceMessage *message = (EphyHistoryServiceMessage*) data;
 
-  g_assert (message->callback);
+  g_assert (message->callback || message->type == CLEAR);
 
   if (g_cancellable_is_cancelled (message->cancellable)) {
     ephy_history_service_message_free (message);
     return FALSE;
   }
-  message->callback (message->service, message->success, message->result, message->user_data);
+
+  if (message->callback)
+    message->callback (message->service, message->success, message->result, message->user_data);
 
   if (message->type == CLEAR)
     g_signal_emit (message->service, signals[CLEARED], 0);
@@ -441,6 +479,35 @@ ephy_history_service_execute_job_callback (gpointer data)
   return FALSE;
 }
 
+typedef struct {
+  EphyHistoryService *service;
+  gpointer user_data;
+  GDestroyNotify destroy_func;
+} SignalEmissionContext;
+
+static void
+signal_emission_context_free (SignalEmissionContext *ctx)
+{
+  g_object_unref (ctx->service);
+  if (ctx->destroy_func && ctx->user_data)
+    ctx->destroy_func (ctx->user_data);
+  g_slice_free (SignalEmissionContext, ctx);
+}
+
+static SignalEmissionContext *
+signal_emission_context_new (EphyHistoryService *service,
+                             gpointer user_data,
+                             GDestroyNotify destroy_func)
+{
+  SignalEmissionContext *ctx = g_slice_new0 (SignalEmissionContext);
+
+  ctx->service = g_object_ref (service);
+  ctx->user_data = user_data;
+  ctx->destroy_func = destroy_func;
+
+  return ctx;
+}
+
 static gboolean
 ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHistoryPageVisit *visit)
 {
@@ -672,6 +739,16 @@ ephy_history_service_query_hosts (EphyHistoryService *self,
   ephy_history_service_send_message (self, message);
 }
 
+static gboolean
+set_url_title_signal_emit (SignalEmissionContext *ctx)
+{
+  EphyHistoryURL *url = (EphyHistoryURL *)ctx->user_data;
+
+  g_signal_emit (ctx->service, signals[URL_TITLE_CHANGED], 0, url->url, url->title);
+
+  return FALSE;
+}
+
 static gboolean
 ephy_history_service_execute_set_url_title (EphyHistoryService *self,
                                             EphyHistoryURL *url,
@@ -684,10 +761,19 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
     g_free (title);
     return FALSE;
   } else {
+    SignalEmissionContext *ctx;
+
     g_free (url->title);
     url->title = title;
     ephy_history_service_update_url_row (self, url);
     ephy_history_service_schedule_commit (self);
+
+    ctx = signal_emission_context_new (self,
+                                       ephy_history_url_copy (url),
+                                       (GDestroyNotify)ephy_history_url_free);
+    g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                     (GSourceFunc)set_url_title_signal_emit,
+                     ctx, (GDestroyNotify)signal_emission_context_free);
     return TRUE;
   }
 }
@@ -758,6 +844,90 @@ ephy_history_service_set_url_zoom_level (EphyHistoryService *self,
   ephy_history_service_send_message (self, message);
 }
 
+static gboolean
+ephy_history_service_execute_set_url_hidden (EphyHistoryService *self,
+                                             EphyHistoryURL *url,
+                                             gpointer *result)
+{
+  gboolean hidden;
+
+  hidden = url->hidden;
+
+  if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
+    /* The URL is not yet in the database, so we can't update it.. */
+    return FALSE;
+  } else {
+    url->hidden = hidden;
+    ephy_history_service_update_url_row (self, url);
+    ephy_history_service_schedule_commit (self);
+    return TRUE;
+  }
+}
+
+void
+ephy_history_service_set_url_hidden (EphyHistoryService *self,
+                                     const char *orig_url,
+                                     gboolean hidden,
+                                     GCancellable *cancellable,
+                                     EphyHistoryJobCallback callback,
+                                     gpointer user_data)
+{
+  EphyHistoryServiceMessage *message;
+  EphyHistoryURL *url;
+
+  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_return_if_fail (orig_url != NULL);
+
+  url = ephy_history_url_new (orig_url, NULL, 0, 0, 0);
+  url->hidden = hidden;
+
+  message = ephy_history_service_message_new (self, SET_URL_HIDDEN,
+                                              url, (GDestroyNotify)ephy_history_url_free,
+                                              cancellable, callback, user_data);
+  ephy_history_service_send_message (self, message);
+}
+
+static gboolean
+ephy_history_service_execute_set_url_thumbnail_time (EphyHistoryService *self,
+                                                     EphyHistoryURL *url,
+                                                     gpointer *result)
+{
+  int thumbnail_time;
+
+  thumbnail_time = url->thumbnail_time;
+
+  if (NULL == ephy_history_service_get_url_row (self, NULL, url))
+    return FALSE;
+  else {
+    url->thumbnail_time = thumbnail_time;
+    ephy_history_service_update_url_row (self, url);
+    ephy_history_service_schedule_commit (self);
+    return TRUE;
+  }
+}
+
+void
+ephy_history_service_set_url_thumbnail_time (EphyHistoryService *self,
+                                             const char *orig_url,
+                                             int thumbnail_time,
+                                             GCancellable *cancellable,
+                                             EphyHistoryJobCallback callback,
+                                             gpointer user_data)
+{
+  EphyHistoryURL *url;
+  EphyHistoryServiceMessage *message;
+
+  g_return_if_fail (EPHY_IS_HISTORY_SERVICE (self));
+  g_return_if_fail (orig_url != NULL);
+
+  url = ephy_history_url_new (orig_url, NULL, 0, 0, 0);
+  url->thumbnail_time = thumbnail_time;
+  message = ephy_history_service_message_new (self, SET_URL_THUMBNAIL_TIME,
+                                              url, (GDestroyNotify) ephy_history_url_free,
+                                              cancellable, callback, user_data);
+  ephy_history_service_send_message (self, message);
+}
+
 static gboolean
 ephy_history_service_execute_get_url (EphyHistoryService *self,
                                       const gchar *orig_url,
@@ -823,6 +993,16 @@ ephy_history_service_get_host_for_url (EphyHistoryService *self,
   ephy_history_service_send_message (self, message);
 }
 
+static gboolean
+delete_urls_signal_emit (SignalEmissionContext *ctx)
+{
+  char *url = (char *)ctx->user_data;
+
+  g_signal_emit (ctx->service, signals[URL_DELETED], 0, url);
+
+  return FALSE;
+}
+
 static gboolean
 ephy_history_service_execute_delete_urls (EphyHistoryService *self,
                                           GList *urls,
@@ -830,10 +1010,18 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
 {
   GList *l;
   EphyHistoryURL *url;
+  SignalEmissionContext *ctx;
 
   for (l = urls; l != NULL; l = l->next) {
     url = l->data;
     ephy_history_service_delete_url (self, url);
+
+    ctx = signal_emission_context_new (self, g_strdup (url->url),
+                                       (GDestroyNotify) g_free);
+    g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                     (GSourceFunc)delete_urls_signal_emit,
+                     ctx,
+                     (GDestroyNotify)signal_emission_context_free);
   }
 
   ephy_history_service_delete_orphan_hosts (self);
@@ -842,15 +1030,34 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
   return TRUE;
 }
 
+static gboolean
+delete_host_signal_emit (SignalEmissionContext *ctx)
+{
+  char *host = (char *)ctx->user_data;
+
+  g_signal_emit (ctx->service, signals[HOST_DELETED], 0, host);
+
+  return FALSE;
+}
+
 static gboolean
 ephy_history_service_execute_delete_host (EphyHistoryService *self,
                                           EphyHistoryHost *host,
                                           EphyHistoryJobCallback callback,
                                           gpointer user_data)
 {
+  SignalEmissionContext *ctx;
+
   ephy_history_service_delete_host_row (self, host);
   ephy_history_service_schedule_commit (self);
 
+  ctx = signal_emission_context_new (self, g_strdup (host->url),
+                                     (GDestroyNotify)g_free);
+  g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                   (GSourceFunc) delete_host_signal_emit,
+                   ctx,
+                   (GDestroyNotify) signal_emission_context_free);
+
   return TRUE;
 }
 
@@ -929,6 +1136,8 @@ ephy_history_service_quit (EphyHistoryService *self,
 static EphyHistoryServiceMethod methods[] = {
   (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_title,
   (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_zoom_level,
+  (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_hidden,
+  (EphyHistoryServiceMethod)ephy_history_service_execute_set_url_thumbnail_time,
   (EphyHistoryServiceMethod)ephy_history_service_execute_add_visit,
   (EphyHistoryServiceMethod)ephy_history_service_execute_add_visits,
   (EphyHistoryServiceMethod)ephy_history_service_execute_delete_urls,
@@ -967,7 +1176,7 @@ ephy_history_service_process_message (EphyHistoryService *self,
   message->result = NULL;
   message->success = method (message->service, message->method_argument, &message->result);
 
-  if (message->callback)
+  if (message->callback || message->type == CLEAR)
     g_idle_add ((GSourceFunc)ephy_history_service_execute_job_callback, message);
   else
     ephy_history_service_message_free (message);
