@@ -3,6 +3,7 @@
  *  Copyright © 2002 Jorn Baayen
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
  *  Copyright © 2003, 2004, 2005, 2006, 2008 Christian Persch
+ *  Copyright © 2012 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,15 +36,13 @@
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
+#include "ephy-string.h"
 #include "ephy-window.h"
 
-#include <errno.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libxml/tree.h>
 #include <libxml/xmlwriter.h>
-#include <stdlib.h>
-#include <string.h>
 
 typedef struct
 {
@@ -58,7 +57,6 @@ typedef struct
 struct _EphySessionPrivate
 {
 	GList *windows;
-	GList *tool_windows;
 	GtkWidget *resume_window;
 
 	GQueue *queue;
@@ -67,8 +65,6 @@ struct _EphySessionPrivate
 	guint dont_save : 1;
 };
 
-#define BOOKMARKS_EDITOR_ID	"BookmarksEditor"
-#define HISTORY_WINDOW_ID	"HistoryWindow"
 #define SESSION_STATE		"type:session_state"
 
 static void ephy_session_iface_init	(EphyExtensionIface *iface);
@@ -264,8 +260,7 @@ session_command_autoresume (EphySession *session,
 
 	if (crashed_session == FALSE ||
 	    policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER ||
-	    priv->windows != NULL ||
-	    priv->tool_windows != NULL)
+	    priv->windows != NULL)
 	{
 		/* If we are auto-resuming, and we never want to
 		 * restore the session, clobber the session state
@@ -425,8 +420,7 @@ session_command_dispatch (EphySession *session)
 			break;
 		case EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW:
 			/* FIXME: maybe just check for normal windows? */
-			if (priv->windows == NULL &&
-			    priv->tool_windows == NULL)
+			if (priv->windows == NULL)
 			{
 				ephy_shell_new_tab_full (ephy_shell_get_default (),
 							 NULL /* window */, NULL /* tab */,
@@ -586,7 +580,6 @@ ephy_session_finalize (GObject *object)
 
 	/* FIXME: those should be NULL already!? */
 	g_list_free (session->priv->windows);
-	g_list_free (session->priv->tool_windows);
 
 	G_OBJECT_CLASS (ephy_session_parent_class)->finalize (object);
 }
@@ -751,39 +744,6 @@ write_window_geometry (xmlTextWriterPtr writer,
 	return ret;
 }
 
-static int
-write_tool_window (xmlTextWriterPtr writer,
-		   GtkWindow *window)
-{
-	const xmlChar *id;
-	int ret;
-
-	if (EPHY_IS_BOOKMARKS_EDITOR (window))
-	{
-		id = (const xmlChar *) BOOKMARKS_EDITOR_ID;
-	}
-	else if (EPHY_IS_HISTORY_WINDOW (window))
-	{
-		id = (const xmlChar *) HISTORY_WINDOW_ID;
-	}
-	else
-	{
-		g_return_val_if_reached (-1);
-	}
-
-	ret = xmlTextWriterStartElement (writer, (const xmlChar *) "toolwindow");
-	if (ret < 0) return ret;
-
-	ret = xmlTextWriterWriteAttribute (writer, (const xmlChar *) "id", id);
-	if (ret < 0) return ret;
-
-	ret = write_window_geometry (writer, window);
-	if (ret < 0) return ret;
-
-	ret = xmlTextWriterEndElement (writer); /* toolwindow */
-	return ret;
-}
-
 static int
 write_ephy_window (xmlTextWriterPtr writer,
 		   EphyWindow *window)
@@ -854,7 +814,7 @@ ephy_session_save (EphySession *session,
 
 	LOG ("ephy_sesion_save %s", filename);
 
-	if (priv->windows == NULL && priv->tool_windows == NULL)
+	if (priv->windows == NULL)
 	{
 		session_delete (session, filename);
 		return TRUE;
@@ -897,12 +857,6 @@ ephy_session_save (EphySession *session,
 	}
 	if (ret < 0) goto out;
 
-	for (w = session->priv->tool_windows; w != NULL && ret >= 0; w = w->next)
-	{
-		ret = write_tool_window (writer, GTK_WINDOW (w->data));
-	}
-	if (ret < 0) goto out;
-
 	ret = xmlTextWriterEndElement (writer); /* session */
 	if (ret < 0) goto out;
 
@@ -976,7 +930,8 @@ parse_embed (xmlNodePtr child,
 			   open an about:blank tab. See http://bugzilla.gnome.org/show_bug.cgi?id=591294
 			   Otherwise, if the web was fully loaded, it is reloaded again. */
 			if (!was_loading ||
-			    strcmp ((const char *) url, "about:blank") == 0)
+			    strcmp ((const char *) url, "about:blank") == 0 ||
+			    strcmp ((const char *) url, "about:overview") == 0)
 			{
 				recover_url = (char *) url;
 				
@@ -1000,8 +955,7 @@ parse_embed (xmlNodePtr child,
 							    EPHY_NEW_TAB_APPEND_LAST);
 				}
 			}
-			else if (was_loading && url != NULL &&
-				 strcmp ((const char *) url, "about:blank") != 0)
+			else if (was_loading)
 			{
 				/* Shows a message to the user that warns that this page was
 				   loading during crash and make Epiphany crash again,
@@ -1018,49 +972,27 @@ parse_embed (xmlNodePtr child,
 	}
 }
 
-static gboolean
-int_from_string (const char *string,
-		 int *retval)
-{
-	char *tail = NULL;
-	long int val;
-	gboolean success = FALSE;
-
-	if (string == NULL) return FALSE;
-
-	errno = 0;
-	val = strtol (string, &tail, 0);
-
-	if (errno == 0 && tail != NULL && tail[0] == '\0')
-	{
-		*retval = (int) val;
-		success = TRUE;
-	}
-
-	return success;
-}
-
 static void
 restore_geometry (GtkWindow *window,
 		  xmlNodePtr node)
 {
 	xmlChar *tmp;
-	int x = 0, y = 0, width = -1, height = -1;
+	gulong x = 0, y = 0, width = 0, height = 0;
 	gboolean success = TRUE;
 
 	g_return_if_fail (window != NULL);
 
 	tmp = xmlGetProp (node, (xmlChar *) "x");
-	success &= int_from_string ((char *) tmp, &x);
+	success &= ephy_string_to_int ((char *) tmp, &x);
 	xmlFree (tmp);
 	tmp = xmlGetProp (node, (xmlChar *) "y");
-	success &= int_from_string ((char *) tmp, &y);
+	success &= ephy_string_to_int ((char *) tmp, &y);
 	xmlFree (tmp);
 	tmp = xmlGetProp (node, (xmlChar *) "width");
-	success &= int_from_string ((char *) tmp, &width);
+	success &= ephy_string_to_int ((char *) tmp, &width);
 	xmlFree (tmp);
 	tmp = xmlGetProp (node, (xmlChar *) "height");
-	success &= int_from_string ((char *) tmp, &height);
+	success &= ephy_string_to_int ((char *) tmp, &height);
 	xmlFree (tmp);
 
 	if (success)
@@ -1074,7 +1006,6 @@ restore_geometry (GtkWindow *window,
 
 		gtk_window_move (window, x, y);
 		gtk_window_set_default_size (window, width, height);
-		
 	}
 }
 
@@ -1163,9 +1094,9 @@ ephy_session_load_from_string (EphySession *session,
 			if (tmp != NULL)
 			{
 				gboolean success;
-				int active_tab;
+				gulong active_tab;
 
-				success = int_from_string ((char *) tmp, &active_tab);
+				success = ephy_string_to_int ((char *) tmp, &active_tab);
 				xmlFree (tmp);
 				if (success)
 				{
@@ -1182,37 +1113,6 @@ ephy_session_load_from_string (EphySession *session,
 				gtk_widget_show (widget);
 			}
 		}
-		else if (xmlStrEqual (child->name, (const xmlChar *) "toolwindow"))
-		{
-			xmlChar *id;
-
-			id = xmlGetProp (child, (const xmlChar *) "id");
-
-			if (id && xmlStrEqual ((const xmlChar *) BOOKMARKS_EDITOR_ID, id))
-			{
-				if (!g_settings_get_boolean
-				    (EPHY_SETTINGS_LOCKDOWN,
-				     EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING))
-				{
-					widget = ephy_shell_get_bookmarks_editor (ephy_shell);
-				}
-			}
-			else if (id && xmlStrEqual ((const xmlChar *) HISTORY_WINDOW_ID, id))
-			{
-				if (!g_settings_get_boolean
-				    (EPHY_SETTINGS_LOCKDOWN,
-				     EPHY_PREFS_LOCKDOWN_HISTORY))
-				{
-					widget = ephy_shell_get_history_window (ephy_shell);
-				}
-			}
-
-			restore_geometry (GTK_WINDOW (widget), child);
-
-			ephy_gui_window_update_user_time (widget, user_time);
-
-			gtk_widget_show (widget);
-		}
 
 		child = child->next;
 	}
@@ -1226,7 +1126,7 @@ ephy_session_load_from_string (EphySession *session,
 
 	g_object_unref (ephy_shell_get_default ());
 
-	return (priv->windows != NULL || priv->tool_windows != NULL);
+	return priv->windows != NULL;
 }
 
 /**
@@ -1292,48 +1192,6 @@ ephy_session_get_windows (EphySession *session)
 	return g_list_copy (session->priv->windows);
 }
 
-/**
- * ephy_session_add_window:
- * @session: a #EphySession
- * @window: a #EphyWindow
- *
- * Add a tool window to the session. #EphyWindow take care of adding
- * itself to session.
- **/
-void
-ephy_session_add_window (EphySession *session,
-			 GtkWindow *window)
-{
-	LOG ("ephy_session_add_window %p", window);
-
-	session->priv->tool_windows =
-		g_list_append (session->priv->tool_windows, window);
-	gtk_application_add_window (GTK_APPLICATION (ephy_shell_get_default ()), window);
-
-	ephy_session_save (session, SESSION_STATE);
-}
-
-/**
- * ephy_session_remove_window:
- * @session: a #EphySession.
- * @window: a #GtkWindow, which must be either the bookmarks editor or the
- * history window.
- *
- * Remove a tool window from the session.
- **/
-void
-ephy_session_remove_window (EphySession *session,
-			    GtkWindow *window)
-{
-	LOG ("ephy_session_remove_window %p", window);
-
-	session->priv->tool_windows =
-		g_list_remove (session->priv->tool_windows, window);
-	gtk_application_remove_window (GTK_APPLICATION (ephy_shell_get_default ()), window);
-
-	ephy_session_save (session, SESSION_STATE);
-}
-
 /**
  * ephy_session_get_active_window:
  * @session: a #EphySession
